package com.arubanetworks.meridiansamples;

import android.graphics.PointF;
import android.os.Bundle;
import android.util.Log;

import com.arubanetworks.meridian.editor.EditorKey;
import com.arubanetworks.meridian.maps.MapFragment;
import com.arubanetworks.meridian.maps.Transaction;
import com.arubanetworks.meridian.requests.MeridianRequest;
import com.arubanetworks.meridian.tags.TagBeacon;
import com.arubanetworks.meridian.tags.TagMarker;
import com.arubanetworks.meridian.tags.TagScan;
import com.arubanetworks.meridian.tags.TagStream;

import java.util.HashMap;
import java.util.List;


/**
 * A Fragment to demonstrate the use of the TagStream for use in displaying Tags.  Demonstrates how to
 * show all tags for a map or a single tag in a location
 */
public class TagsFragment extends MapFragment implements TagStream.Listener {
    private static final String TAG_MAC = "meridian.TAG_MAC";
    // A collection to maintain our TagMarkers
    private final HashMap<TagBeacon, TagMarker> tagMarkers = new HashMap<>();

    // The TagStream itself
    private TagStream tagStream;
    private String tagMac;

    public static TagsFragment newInstance(EditorKey mapKey) {
        TagsFragment f = new TagsFragment();
        f.setArguments(new MapFragment.Builder().setMapKey(mapKey).build().getArguments());
        return f;
    }

    public static TagsFragment newInstance(EditorKey mapKey, String tagMac) {
        TagsFragment f = new TagsFragment();
        Bundle args = new MapFragment.Builder().setMapKey(mapKey).build().getArguments();
        if (args != null) {
            args.putString(TAG_MAC, tagMac);
            f.setArguments(args);
        }
        return f;
    }

    @Override
    public void onMapLoadStart() {
        super.onMapLoadStart();
        // Close out any existing tags when loading a new map key
        if (tagStream != null) tagStream.stopUpdatingTags();

        // Clear any references to old markers so we don't re-add them to the new map on accident
        tagMarkers.clear();

        // Create our TagStream and start it.
        if (tagMac != null) {
            // only listen for specified tag
            tagStream = new TagStream.Builder()
                    .addTagMac(Application.APP_KEY, tagMac)
                    .setListener(this)
                    .build();
        } else {
            tagStream = new TagStream.Builder()
                    .addMapKey(getMapView().getMapKey())
                    .setListener(this)
                    .build();
        }
    }

    @Override
    public void onTagsUpdated(List<TagBeacon> tags) {

        Transaction.Builder transaction = new Transaction.Builder().setType(Transaction.Type.UPDATE);

        // Iterate through the new or updated tags and add them to the map.
        for (TagBeacon tag : tags) {
            // ignore tags that are not on the current map
            if (getMapView() != null && getMapView().getMapKey() != null && !tag.getMapId().equals(getMapView().getMapKey().getId())) continue;
            TagMarker currentMarker = tagMarkers.get(tag);

            // check for a new Tag and add
            if (currentMarker == null) {
                if (getContext() == null) {return;}
                currentMarker = new TagMarker(getContext(), tag);
                tagMarkers.put(tag, currentMarker);
            } else {
                // Otherwise update our current marker to reflect the new tag state
                PointF point = tag.getLocation().getPoint();
                currentMarker.setPosition(point.x, point.y);

                // The title or image could also have changed but the TagMarker provided with the
                // SDK doesn't show those fields by default so we can ignore those changes here.
            }
            transaction.addMarker(currentMarker);
        }

        // Commit the transaction
        if (transaction.getSize() > 0)
            getMapView().commitTransaction(transaction.build());
    }

    @Override
    public void onTagsRemoved(List<TagBeacon> tags) {
        Transaction.Builder transaction = new Transaction.Builder().setType(Transaction.Type.REMOVE);

        for (TagBeacon tag : tags) {
            TagMarker currentMarker = tagMarkers.get(tag);

            // We can ignore removing the tags marker if we never added it.
            if (currentMarker == null) continue;

            transaction.addMarker(currentMarker);
        }
        getMapView().commitTransaction(transaction.build());
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (getArguments() != null) {
            tagMac = getArguments().getString(TAG_MAC);
        }
    }

    @Override
    public void onPause() {
        super.onPause();
        // If we are paused we stop listening to tag updates
        if (tagStream != null) tagStream.stopUpdatingTags();
    }

    @Override
    public void onResume() {
        super.onResume();
        // If we are resuming, start listening for tags again.
        if (tagStream != null) tagStream.startUpdatingTags();

        /*// example how to scan for tags that are nearby.
        TagScan ts = new TagScan();
        ts.findNearbyTags(
                Application.APP_KEY,
                Application.TAG_MAC, // return specified tag or all tags if null
                0, // use default scan length (12s)
                0, // don't filter by RSSI
                new MeridianRequest.Listener<List<TagBeacon>>() {
                    @Override
                    public void onResponse(List<TagBeacon> response) {
                        if (response != null) {
                            Log.d("TagsFragment", "Found " + response.size() + " nearby tags");
                            for (TagBeacon tag : response) {
                                Log.d("TagsFragment", "Found:" + tag.toString());
                            }
                        }
                    }
                },
                null
        );*/
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        // Clean up memory.
        if (tagStream != null) tagStream.dispose();
    }
}
